@@ -222,6 +222,10 @@ bool AAptInterface::cancel()
 		iTerminated = true;
 		return true;
 	}
+	if( iMode == ModeFetchDates ) {
+		iTerminated = true;
+		return true;
+	}
 	return false;
 }
 
@@ -301,7 +305,7 @@ bool AAptInterface::startAptGetInstall()
 
 	iProcAptGetInstallOutput.clear();
 
-	//qDebug() << "running apt-get install";
+	qDebug() << "running apt-get install";
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
@@ -446,11 +450,7 @@ QString AAptInterface::finishProcessCommonErrorMessages(QByteArray& output)
 	} else if( output.contains("E: Unable to fetch some archives") ) {
 		msg = "Failed to fetch packages - Your network connection might be down or your catalogs could be out of date";
 	} else if( output.contains("E: Couldn't find package") ) {
-		int pos = output.indexOf("E: Couldn't find package");
-		msg = "Missing package: ";
-		if( pos!=-1 )
-			msg += output.mid(pos) + " - ";
-		msg += "Your catalogs might be out of date";
+		msg = "Missing package, see the log for details - Your catalogs might be out of date";
 	} else if( output.contains("E: Broken packages") ) {
 		msg = "Your system has broken packages, see the log for details";
 	} else if( output.contains("E: Handler silently failed") ) {
@@ -461,6 +461,8 @@ QString AAptInterface::finishProcessCommonErrorMessages(QByteArray& output)
 		msg = "DNS errors were reported, check your network connection and/or repository configuration";
 	} else if( output.contains("E: dpkg was interrupted") ) {
 		msg = "Your system has partially installed or broken packages. You'll have to fix this manually. Try dpkg --configure -a";
+	} else if( output.contains("dpkg: error processing") || output.contains("Errors were encountered while processing:") ) {
+		msg = "dpkg reported errors while processing a package - see the log for details";
 	}
 
 	return msg;
@@ -497,9 +499,23 @@ void AAptInterface::finishedAptGetUpdate(int exitCode, QProcess::ExitStatus exit
 		msg = "Failed to update some or all of the catalogs. Check your network connection and/or repository configuration";
 	}
 
-	if( success )
+	if( success ) {
 		iNeedRepoRefresh = false;
 
+		QFile lastupdate("/root/.fapman/lastupdate");   // create an empty file and/or update the modification time
+		if( lastupdate.open(QIODevice::WriteOnly) )
+			lastupdate.close();
+
+		int pos = iProcAptGetUpdateOutput.indexOf("\nFetched ");
+		if( pos!=-1 ) {
+			msg += "<br>apt-get: ";
+			msg += iProcAptGetUpdateOutput.mid(pos+1, iProcAptGetUpdateOutput.indexOf('\n', pos+1)-pos ).trimmed();
+		}
+
+	} else {
+		cleanAfterError();
+	}
+
 	iProcAptGetUpdate->close();
 	communicateStatusToUi(success, title, msg);
 	runNext();
@@ -569,6 +585,8 @@ void AAptInterface::finishedAptGetSimulate(int exitCode, QProcess::ExitStatus ex
 			}
 		}
 	}
+	if( !success )
+		cleanAfterError();
 
 	iProcAptGetSimulate->close();
 	communicateStatusToUi(success, title, msg);
@@ -608,6 +626,9 @@ void AAptInterface::finishedAptGetInstall(int exitCode, QProcess::ExitStatus exi
 	iProcessPackagesOrig.clear();
 	iProcessPackageVersions.clear();
 
+	if( !success )
+		cleanAfterError();
+
 	iProcAptGetInstall->close();
 	communicateStatusToUi(success, title, msg);
 	runNext();
@@ -634,6 +655,8 @@ void AAptInterface::finishedAptGetClean(int exitCode, QProcess::ExitStatus exitS
 		title = "Error";
 		msg = finishProcessCommonErrorMessages(output);
 	}
+	if( !success )
+		cleanAfterError();
 
 	iProcAptGetClean->close();
 	communicateStatusToUi(success, title, msg);
@@ -760,7 +783,7 @@ void AAptInterface::progressCheckTimerCallback()
 
 void AAptInterface::communicateStatusToUi(bool success, QString title, QString msg)
 {
-	//qDebug() << title << msg;
+	qDebug() << title << msg;
 	iQueueMessages.append(msg);
 
 	if( iMainWindow && iOperationsQueue.count()==0 )
@@ -911,10 +934,11 @@ bool AAptInterface::writeRepositories()
 		QTextStream out(&ham);
 		for( int i=0; i<iRepositories.count(); i++ )
 			out << iRepositories.at(i)->toString() << "\n";
-
 		ham.close();
 
 		return true;
+	} else {
+		qDebug() << "Failed to write repository list to sources.d!";
 	}
 
 	return false;
@@ -929,13 +953,21 @@ bool AAptInterface::needRepoRefresh()
 	if( iNeedRepoRefresh || iSettings->qsettings()->value("need_repo_refresh", false).toBool() ) {
 		iNeedRepoRefresh = false;
 		iSettings->qsettings()->setValue("need_repo_refresh", false);
+		qDebug() << "repo update required, debug 1";
 		return true;
 	}
 
 	QFile own("/root/.fapman/repos.list");
 	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
 
-	if( own.size() != ham.size() ) {
+	if( own.exists() && ham.exists() )
+	{
+		if( own.size() != ham.size() ) {
+			qDebug() << "repo update required, debug 2";
+			return true;
+		}
+	} else {
+		qDebug() << "repo update required, debug 3";
 		return true;
 	}
 
@@ -953,9 +985,11 @@ bool AAptInterface::needRepoRefresh()
 	bDate = bDate.addSecs(24*60*60); //24h
 
 	if( aDate < QDateTime::currentDateTime() && bDate < QDateTime::currentDateTime() ) {
+		qDebug() << "repo update required, debug 4";
 		return true;
 	}
 
+	qDebug() << "repo update not required";
 	return false;
 }
 
@@ -964,7 +998,7 @@ bool AAptInterface::needRepoRefresh()
 void AAptInterface::startPkgListRead()
 {
 	logToFile( QString("Start reading package lists") );
-	//qDebug() << "reading package list files";
+	qDebug() << "reading package list files";
 
 	if( iUiDimmer ) {
 		iUiDimmer->updateText("Reading package lists<br>");
@@ -1006,12 +1040,14 @@ void AAptInterface::startPkgListRead()
 	}
 	QFileInfo dbinfo("/var/lib/dpkg/status");
 	totaldatasize += dbinfo.size();
+	int filecount = 0;
 
 	for( int i=0; i<files.count(); i++ )
 	{
 		Repository* currentRepo = 0;
 		if( files.at(i).absoluteFilePath().endsWith("_Packages") )
 		{
+			filecount++;
 			for(int x=0; x<iRepositories.count(); x++) {
 				if( iRepositories.at(x)->toListFileNames().contains( files.at(i).fileName() ) ) {
 					currentRepo = iRepositories.at(x);
@@ -1067,11 +1103,14 @@ void AAptInterface::startPkgListRead()
 			db.close();
 		}
 	}
+
+	qDebug() << "Processed" << filecount << "package list files";
+
 	if( iTerminated ) {
 		if( iUiDimmer )
 			iUiDimmer->setProgress(-1);
-		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
 		cleanAfterError();
+		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");		
 		return;
 	}
 
@@ -1108,11 +1147,9 @@ void AAptInterface::startPkgListRead()
 	}
 	db.close();
 
-	/*
-	qDebug() << "Processed" << pkgcount_apt << "(apt) and" << pkgcount_dpkg << "(dpkg) packages";
+	qDebug() << "Processed" << pkgcount_apt << "(apt) and" << pkgcount_dpkg << "(dpkg) package entries";
 	qDebug() << "In DB:" << iPackagesAvailable.count() << "packages available,"
 			<< iPackagesInstalled.count() << "installed";
-	*/
 
 	logToFile( QString("Finished reading package lists") );
 	if( iUiDimmer ) {
@@ -1122,11 +1159,13 @@ void AAptInterface::startPkgListRead()
 	}
 
 	if( iTerminated ) {
-		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
 		cleanAfterError();
+		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
 		return;
 	}
 
+	readBlacklist();
+
 	communicateStatusToUi(true, "Operation finished", "Package data read");
 	iCanCancel = false;
 	runNext();
@@ -1149,7 +1188,8 @@ Package* AAptInterface::ReadNextPackage(QFile& f, quint64& currentreaddata)
 
 	if( pkg->name() != "" && pkg->isInstalled() ) {
 		QFileInfo f( "/var/lib/dpkg/info/" + pkg->name() + ".list" );
-		pkg->setDate( f.lastModified() );
+		if( f.exists() )
+			pkg->setDate( f.lastModified() );
 	}
 
 	pkg->updateStatus();
@@ -1240,71 +1280,353 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 }
 
 
-void AAptInterface::startFetchDates()
+
+void AAptInterface::writeBlacklist()
 {
-	logToFile( QString("Start fetching package dates") );
-
-	//if( iUiDimmer ) {
-	//	iUiDimmer->updateText("Fetching package date information");
-	//}
-
-	QNetworkAccessManager* nam = new QNetworkAccessManager();
-
-	//connect(nam,SIGNAL(finished(QNetworkReply*)),this,SLOT(dateFetchNetworkReply(QNetworkReply*)));
-
 	QHashIterator<QString, Package*> i( iPackagesAvailable );
-	QString fn,fnf,datecmp;
-	int count=0;
 	while (i.hasNext())
 	{
 		i.next();
-		count++;
-		iUiDimmer->updateText("Fetching package date information<br>" + QString("%1/%2").arg(count).arg(iPackagesAvailable.count()));
-		if( i.value()->repositories().at(0) && i.value()->fileNameFull()!="" ) {
-			int x = i.value()->fileNameFull().lastIndexOf('/');
-			fnf = i.value()->fileName();
-			if( x!=-1) {
-				fn = i.value()->fileNameFull().left(x);
-			} else {
-				fn = i.value()->fileNameFull();
-			}
 
-			datecmp = "<a href=\"" + fnf + "\">" + fnf + "</a>";
-
-			QNetworkRequest nwr(QUrl(i.value()->repositories().at(0)->url() + i.value()->repositories().at(0)->dir()
-									 + fn + '/'));
-
-			//qDebug() << nwr.url();
-			//qDebug() << datecmp;
-
-			QNetworkReply* reply = nam->get(nwr);
-
-			while(reply->isRunning()) {
-				QApplication::processEvents();
-				usleep(10000);
-			}
-
-			QByteArray data = reply->readAll();
-			delete reply;
-			//qDebug() << data;
-
-			int p = data.indexOf(datecmp);
-			//qDebug() << p;
-			QString k = "";
-			if( p!=-1 ) {
-				int z = data.indexOf('<', p+datecmp.length()+1);
-				if( z != -1 ) {
-					k = data.mid(p+datecmp.length()+1, z-(p+datecmp.length()+1) );
-				}
-			}
-			qDebug() << i.value()->name() << k.trimmed();
+		if( i.value()->blacklisted() == BlacklistSelect::BlacklistAll ) {
+			iBlacklist << i.value()->name();
+		}
+		else if( i.value()->blacklisted() == BlacklistSelect::BlacklistThis ) {
+			iBlacklist << (i.value()->name() + " " + i.value()->version());
 		}
 	}
 
+	iBlacklist.removeDuplicates();
+
+	QFile f("/root/.fapman/black.list");
+	if( f.open(QIODevice::WriteOnly | QIODevice::Text ) )
+	{
+		QTextStream out(&f);
+		for( int i=0; i<iBlacklist.count(); i++ )
+			out << iBlacklist.at(i) << "\n";
+		f.close();
+	}
+
+	qDebug() << "blacklist: wrote" << iBlacklist.count() << "entries";
+}
+
+void AAptInterface::readBlacklist()
+{
+	QFile f("/root/.fapman/black.list");
+	if( f.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while( !f.atEnd() ) {
+			QString line = f.readLine().trimmed();
+			if( line != "" )
+				iBlacklist.append(line);
+		}
+		f.close();
+	}
+
+	iBlacklist.removeDuplicates();
+
+	qDebug() << "blacklist: read" << iBlacklist.count() << "entries";
+
+	for( int i=0; i<iBlacklist.count(); i++ )
+	{
+		QStringList parts = iBlacklist.at(i).split(' ');
+		if( parts.count()==1 ) {
+			Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
+			if( pkg ) {
+				pkg->setBlacklisted(BlacklistSelect::BlacklistAll);
+			}
+		} else if( parts.count()==2 ) {
+			Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
+			if( pkg && pkg->version()==parts.at(1) ) {
+				pkg->setBlacklisted(BlacklistSelect::BlacklistThis);
+			}
+		} else {
+			qDebug() << "Warning: invalid blacklist entry:" << iBlacklist.at(i);
+		}
+	}
+}
+
+void AAptInterface::removeFromBlacklist(Package *pkg, BlacklistSelect::blackList oldstate)
+{
+	if( !pkg ) {
+		qDebug() << "Warning: trying to remove null package from blacklist";
+		return;
+	}
+
+	QStringList newlist;
+	bool removed = false;
+
+	for( int i=0; i<iBlacklist.count(); i++ )
+	{
+		if( oldstate == BlacklistSelect::BlacklistAll )
+		{
+			if( !(iBlacklist.at(i) == pkg->name()) ) {
+				newlist << iBlacklist.at(i);
+			} else removed = true;
+		} else if( oldstate == BlacklistSelect::BlacklistThis ) {
+			if( !(iBlacklist.at(i) == (pkg->name()+" "+pkg->version())) ) {
+				newlist << iBlacklist.at(i);
+			} else removed = true;
+		}
+	}
+
+	if( removed )
+		qDebug() << "blacklist: removed" << pkg->name();
+	else
+		qDebug() << "blacklist:" << pkg->name() << "not in saved list";
+
+	iBlacklist = newlist;
+}
+
+
+void AAptInterface::startFetchDates()
+{
+	logToFile( QString("Start fetching package dates") );
+	qDebug() << "start fetching package dates";
+
+	if( iUiDimmer ) {
+		iUiDimmer->updateText("Reading date cache");
+		iUiDimmer->setProgress(0);
+		QApplication::processEvents();
+	}
+
+	readDateCache();
+
+	if( iUiDimmer ) {
+		iUiDimmer->updateText("Fetching package date information");
+		QApplication::processEvents();
+	}
+
+	QNetworkAccessManager* nam = new QNetworkAccessManager(this);
+	iCanCancel = true;
+
+	int count = 0;
+	int updProgress = 0;
+
+	QHash<QString, Package*> fetchable;
+	QHashIterator<QString, Package*> i( iPackagesAvailable );
+	while (i.hasNext() )
+	{
+		i.next();
+		if( !i.value()->date().isValid() && i.value()->section().startsWith("user/") )
+		{
+			Repository* repo = 0;
+			for( int x=0; x<i.value()->repositories().count(); x++ )
+				if( i.value()->repositories().at(x) && i.value()->repositories().at(x)->url().startsWith("http://repository.maemo.org") )
+				{
+				repo = i.value()->repositories().at(x);
+				break;
+			}
+			if( repo ) {
+				fetchable.insert(i.value()->name(), i.value());
+			}
+		}
+	}
+
+	connect(nam, SIGNAL(finished(QNetworkReply*)), this, SLOT(dateFetchNetworkReply(QNetworkReply*)));
+	iDateRequestsWaiting = 0;
+	iDateRequestsSent = 0;
+	iDateRequestsReceived = 0;
+	iNetworkError = QNetworkReply::NoError;
+
+	qDebug() << "dates to fetch:" << fetchable.count();
+
+	QHashIterator<QString, Package*> fe( fetchable );
+	while (fe.hasNext() && !iTerminated)
+	{
+		fe.next();
+
+		if( updProgress >=10 ) {
+			iUiDimmer->setProgress( count*100/fetchable.count() );
+			updProgress=0;
+		}
+
+		if( !fe.value()->date().isValid() && fe.value()->section().startsWith("user/") )
+		{
+			QString url;
+			Repository* repo = 0;
+			for( int x=0; x<fe.value()->repositories().count(); x++ )
+				if( fe.value()->repositories().at(x) && fe.value()->repositories().at(x)->url().startsWith("http://repository.maemo.org") )
+				{
+				repo = fe.value()->repositories().at(x);
+				break;
+			}
+
+			if( repo )
+			{
+				//qDebug() << "getting date for" << fe.value()->name();
+
+				url = repo->url() + repo->dir() + fe.value()->fileNameFull();
+				//qDebug() << url;
+
+				QUrl u(url);
+				QNetworkRequest r(u);
+
+				if( iDateRequestsReceived == 0 ) {
+					while( iDateRequestsWaiting>0 ) {
+						QApplication::processEvents(QEventLoop::WaitForMoreEvents);
+					}
+				} else {
+					while( iDateRequestsWaiting>50 ) {
+						QApplication::processEvents(QEventLoop::WaitForMoreEvents);
+					}
+				}
+
+				if( iDateRequestsReceived>0 && iNetworkError != QNetworkReply::NoError ) {
+					qDebug() << "network error" << iNetworkError;
+					break;
+				} else {
+					nam->head(r);
+					iDateRequestsSent++;
+					iDateRequestsWaiting++;
+				}
+
+				/*QNetworkReply* reply = nam->head(r);
+
+				while(reply->isRunning()) {
+					QApplication::processEvents(QEventLoop::WaitForMoreEvents);
+				}
+
+				//qDebug() << "reply" << reply->header(QNetworkRequest::LastModifiedHeader).toDateTime();
+				if( reply->header(QNetworkRequest::LastModifiedHeader).toDateTime().isValid() )
+					fe.value()->setDate( reply->header(QNetworkRequest::LastModifiedHeader).toDateTime() );
+
+				delete reply;*/
+			}
+			count = iDateRequestsReceived;
+			updProgress++;
+		}
+	}
+	while( iDateRequestsWaiting>0 ) {
+		if( updProgress >=10 ) {
+			iUiDimmer->setProgress( count*100/fetchable.count() );
+			updProgress=0;
+		}
+		QApplication::processEvents(QEventLoop::WaitForMoreEvents);
+		count = iDateRequestsReceived;
+		updProgress++;
+	}
 	delete nam;
 
-	logToFile( QString("Finished fetching package dates") );
+	if( iNetworkError != QNetworkReply::NoError )
+	{
+		// don't stop on this error, only inform the user
+		iMainWindow->notifyDialog("Network error", "There was a network error while fetching date information");
+	}
+
+	if( iTerminated ) {
+		if( iUiDimmer )
+			iUiDimmer->setProgress(-1);
+		cleanAfterError();
+		communicateStatusToUi(false, "Operation cancelled", "The operation was cancelled by user");
+		return;
+	}
+	iCanCancel = false;
+
+	if( iUiDimmer ) {
+		iUiDimmer->updateText("Writing date cache");
+		iUiDimmer->setProgress(100);
+		QApplication::processEvents();
+	}
+
+	qDebug() << "sent" << iDateRequestsSent << "requests, received" << iDateRequestsReceived << "replies with"
+			<< iDateRequestErrors << "errors";
+
+	logToFile( QString("Finished fetching package dates, writing cache") );
+	writeDateCache();
+
+	if( iUiDimmer ) {
+		iUiDimmer->updateText("Creating package view");
+		QApplication::processEvents();
+		iUiDimmer->setProgress(-1);
+	}
 
 	communicateStatusToUi(true, "Operation finished", "Package dates fetched");
 	runNext();
 }
+
+void AAptInterface::dateFetchNetworkReply(QNetworkReply* reply)
+{
+	iDateRequestsWaiting--;
+	iDateRequestsReceived++;
+	//qDebug() << "reply" << reply->header(QNetworkRequest::LastModifiedHeader).toDateTime();
+	iNetworkError = reply->error();
+
+	if( reply->header(QNetworkRequest::LastModifiedHeader).toDateTime().isValid() &&
+		iNetworkError == QNetworkReply::NoError)
+	{
+		QString path = reply->url().path();
+		int pos = path.lastIndexOf('/');
+		int pos2 = path.indexOf('_', pos);
+		if( pos!=-1 && pos2!=-1 ) {
+			QString pkgname = path.mid(pos+1,pos2-pos-1);
+			Package* pkg = iPackagesAvailable.value(pkgname, 0);
+			if( pkg ) {
+				pkg->setDate( reply->header(QNetworkRequest::LastModifiedHeader).toDateTime() );
+			}
+		}
+	}
+
+	if( iNetworkError != QNetworkReply::NoError )
+		iDateRequestErrors++;
+	reply->deleteLater();
+}
+
+void AAptInterface::writeDateCache()
+{
+	qDebug() << "writing date cache";
+
+	QFile f("/root/.fapman/dates.cache");
+	if( f.open(QIODevice::WriteOnly | QIODevice::Text ) )
+	{
+		QTextStream out(&f);
+		QHashIterator<QString, Package*> i( iPackagesAvailable );
+		while (i.hasNext() )
+		{
+			i.next();
+			if( i.value()->date().isValid() && i.value()->section().startsWith("user/") )
+				out << i.value()->name() << " " << i.value()->version() << " " << i.value()->date().toString(Qt::ISODate) << "\n";
+		}
+		f.close();
+	} else {
+		qDebug() << "Warning: failed to write date cache";
+	}
+}
+
+void AAptInterface::readDateCache()
+{
+	qDebug() << "reading date cache";
+
+	QFile f("/root/.fapman/dates.cache");
+	if( f.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while( !f.atEnd() ) {
+			QString line = f.readLine().trimmed();
+			QStringList parts = line.split(' ');
+			if( parts.count()==3 ) {
+				Package* pkg = iPackagesAvailable.value(parts.at(0),0);
+				if( pkg && pkg->section().startsWith("user/") && pkg->version()==parts.at(1) )
+				{
+					QDateTime dt = QDateTime::fromString( parts.at(2), Qt::ISODate);
+					if( dt.isValid() ) {
+						pkg->setDate( dt );
+					} else {
+						qDebug() << "Warning: Invalid date in date cache";
+					}
+				}
+			} else {
+				qDebug() << "Warning: error in date cache:" << line;
+			}
+		}
+		f.close();
+	} else {
+		qDebug() << "date cache does not exist";
+	}
+}
+
+bool AAptInterface::dateCacheExists()
+{
+	QFileInfo f("/root/.fapman/dates.cache");
+
+	return f.exists();
+}
