@@ -17,18 +17,20 @@
 	(C) Heikki Holstila 2010
 */
 
+#ifdef MYDEF_GTK_EXISTS
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+#endif
+
 #include <QtCore>
 #include <QtGui>
-#include <iostream>
+#include <QDBusConnection>
+#include <QDBusInterface>
 
 #ifdef Q_WS_MAEMO_5
 #include <QtMaemo5>
 #endif
 
-#ifdef MYDEF_GTK_EXISTS
-#include <gdk/gdk.h>
-#endif
-
 #include "mainwindow.h"
 #include "version.h"
 #include "ui_mainwindow.h"
@@ -77,6 +79,19 @@ MainWindow::MainWindow(QWidget *parent) :
 	// clear log on startup
 	QFile logfile(KLogFileName);
 	logfile.remove();
+
+	/*
+	// does not work
+
+	QDBusConnection conn = QDBusConnection::connectToBus(QDBusConnection::SystemBus, "faster_application_manager");
+
+	QString service = "com.nokia.icd";
+	QString path = "/com/nokia/icd";
+	QString method = "connect";
+
+	QDBusInterface net(service, path, service, conn, this);
+	net.call(method,"[ANY]",0);
+	*/
 }
 
 MainWindow::~MainWindow()
@@ -114,13 +129,70 @@ void MainWindow::on_btnUpdate_clicked()
 	// update catalogs
 
 	busyDialog(true, tr("Operation in progress"), tr("Updating catalogs"));
+
 	iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
 	iAptInterface->run(iDimmer);
 }
 
+void MainWindow::on_btnListInstallable_clicked()
+{
+	//install
+
+	iWinPackageView->setStatFilter( Package::PkgStatNotInstalled );
+
+	if( iAptInterface->needRepoRefresh() ) //&& !iSettings->qsettings()->value("never_auto_update", false).toBool() )
+		iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
+
+	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
+
+	iNextOperation = OpOpenPkgView;
+	iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
+
+	if( iSettings->qsettings()->value("fetch_dates",false).toBool() )
+	{
+		if( !iAptInterface->dateCacheExists() )
+		{
+			ConfirmDialog d(false, this);
+			d.setText("Notice","Date information will be fetched only for packages from maemo.org and only for user categories.");
+			d.exec();
+		}
+		iAptInterface->addQueuedOperation(AAptInterface::ModeFetchDates);
+	}
+
+	iAptInterface->run(iDimmer);
+}
+
+void MainWindow::on_btnUpgrade_clicked()
+{
+	// upgrade
+
+	iWinPackageView->setStatFilter( Package::PkgStatUpgradeable );
+
+	if( iAptInterface->needRepoRefresh() ) //&& !iSettings->qsettings()->value("never_auto_update", false).toBool() )
+		iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
+
+	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
+
+	iNextOperation = OpOpenPkgView;
+	iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
+
+	if( iSettings->qsettings()->value("fetch_dates",false).toBool() )
+	{
+		if( !iAptInterface->dateCacheExists() )
+		{
+			ConfirmDialog d(false, this);
+			d.setText("Notice","Date information will be fetched only for packages from maemo.org and only for user categories.");
+			d.exec();
+		}
+		iAptInterface->addQueuedOperation(AAptInterface::ModeFetchDates);
+	}
+
+	iAptInterface->run(iDimmer);
+}
+
 void MainWindow::on_btnListInstalled_clicked()
 {
-	//uninstall
+	//remove
 
 	iWinPackageView->setStatFilter( Package::PkgStatInstalled );
 
@@ -133,42 +205,7 @@ void MainWindow::on_btnListInstalled_clicked()
 
 	iNextOperation = OpOpenPkgView;
 	iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
-	iAptInterface->run(iDimmer);
-}
 
-void MainWindow::on_btnListInstallable_clicked()
-{
-	//install
-
-	iWinPackageView->setStatFilter( Package::PkgStatNotInstalled );
-
-	if( iAptInterface->needRepoRefresh() )
-		iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
-
-	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
-
-	iNextOperation = OpOpenPkgView;
-	iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
-
-	// TESTING
-	//iAptInterface->addQueuedOperation(AAptInterface::ModeFetchDates);
-
-	iAptInterface->run(iDimmer);
-}
-
-void MainWindow::on_btnUpgrade_clicked()
-{
-	// upgrade
-
-	iWinPackageView->setStatFilter( Package::PkgStatUpgradeable );
-
-	if( iAptInterface->needRepoRefresh() )
-		iAptInterface->addQueuedOperation(AAptInterface::ModeAptGetUpdate);
-
-	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
-
-	iNextOperation = OpOpenPkgView;
-	iAptInterface->addQueuedOperation(AAptInterface::ModeReadPackages);
 	iAptInterface->run(iDimmer);
 }
 
@@ -371,12 +408,21 @@ void MainWindow::on_actionOptions_triggered()
 	iSettings->openWin();
 }
 
+void MainWindow::notifyDialog(QString title, QString msg)
+{
+	ConfirmDialog d(false, this);
+	d.setText(title, msg);
+	d.exec();
+}
+
 void MainWindow::GdkEventIconThemeReload()
 {
-	// it appears this does not work very well
+	// DOES NOT EVEN WORK (at least not reliably) - disabled from the project file
 
 #ifdef MYDEF_GTK_EXISTS
-	//qDebug() << "Sending GDK client icon theme reload event";
+	qDebug() << "Sending GDK icon theme reload event";
+
+	gdk_init((int*)QApplication::argc(),(gchar***)QApplication::argv());
 
 	GdkEventClient ev;
 	ev.type = GDK_CLIENT_EVENT;
@@ -385,5 +431,10 @@ void MainWindow::GdkEventIconThemeReload()
 	ev.message_type = gdk_atom_intern_static_string("_GTK_LOAD_ICONTHEMES");
 	ev.data_format = 32;
 	gdk_event_send_clientmessage_toall((GdkEvent*)&ev);
+
+	while(gdk_events_pending()) {
+		g_main_context_iteration(NULL, true);
+	}
+
 #endif
 }
