@@ -21,6 +21,8 @@
 #define PACKAGE_H
 
 #include <QtCore>
+#include "blacklistselect.h"
+
 class AAptInterface;
 class Repository;
 
@@ -49,6 +51,7 @@ public:
 	void updateStatus();
 	void addRepository(Repository* r_) { iRepositories.append(r_); }
 	void setDate(QDateTime d_) { iDate=d_; }
+	void setBlacklisted(BlacklistSelect::blackList bl_) { iBlacklist=bl_; }
 
 	void convertIcon();
 
@@ -74,6 +77,8 @@ public:
 	QString fileNameFull() { return iFileNameFull; }
 	QDateTime date() { return iDate; }
 	QList<Repository*> repositories() { return iRepositories; }
+	BlacklistSelect::blackList blacklisted() { return iBlacklist; }
+	bool isBlacklisted() { if(iBlacklist==BlacklistSelect::BlacklistNone) return false; else return true;}
 
 	static bool versionCompare(QString isNewer, QString compare);
 
@@ -95,10 +100,10 @@ private:
 	QString iFileNameFull;
 	QDateTime iDate;
 	QList<Repository*> iRepositories;
+	BlacklistSelect::blackList iBlacklist;
 
 	QByteArray iIconData;
 	QPixmap* iIcon;
-
 };
 
 #endif // PACKAGE_H
