@@ -17,17 +17,30 @@
 	(C) Heikki Holstila 2010
 */
 
+#include <QDBusConnection>
+#include <QDBusInterface>
 #include "packageselector.h"
 #include "ui_packageselector.h"
 #include "package.h"
 #include "packageview.h"
 #include "repository.h"
+#include "blacklistselect.h"
+#include "aaptinterface.h"
 
-PackageSelector::PackageSelector(Package* pkg, QWidget *parent) :
+PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *parent) :
     QDialog(parent),
     ui(new Ui::PackageSelector)
 {
-    ui->setupUi(this);
+    ui->setupUi(this);	
+	iPkg = pkg;
+	iAptInterface = apt;
+
+	ui->pushButton_website->setIcon(QPixmap("/usr/share/icons/hicolor/48x48/hildon/general_web.png"));
+	if( getMaemoOrgUrl(pkg)=="" && getMaemoOrgUrl(pkg->availablePackage())=="" ) {
+		ui->pushButton_website->setEnabled(false);
+	}
+	if( pkg->isInstalled() || pkg->isMarkedForOperation() )
+		ui->pushButton_blacklist->setEnabled(false);
 
 	if( pkg->icon() && !pkg->icon()->isNull() ) {
 		ui->label_appicon->setPixmap( *pkg->icon() );
@@ -41,19 +54,12 @@ PackageSelector::PackageSelector(Package* pkg, QWidget *parent) :
 	else
 		ui->label_header->setText("<b>" + pkg->name() + "</b>");
 
+	updateHeader2();
+
 	QString longtext;
 
 	longtext += "Category: " + pkg->section() + "<br>";
 
-	if( pkg->isInstalled() ) {
-		longtext += "Package is installed, version <b>" + pkg->version() + "</b>";
-		if( pkg->installedSize() > 0 )
-			longtext += QString(" (%L1 kB)").arg(pkg->installedSize() );
-	} else {
-		longtext += "Package is not installed";
-	}
-	longtext += "<br>";
-
 	if( pkg->isInstalled() ) {
 		longtext += "Installation date: ";
 		if( pkg->date().isValid() )
@@ -63,6 +69,10 @@ PackageSelector::PackageSelector(Package* pkg, QWidget *parent) :
 		longtext += "<br>";
 	}
 
+	if( pkg->isInstalled() && pkg->installedSize() > 0 )
+		longtext += QString("Size: %L1 kB<br>").arg(pkg->installedSize() );
+
+
 	if( !pkg->isInstalled() ){
 		longtext += "Available version <b>" + pkg->version() + "</b>";
 		if( pkg->size() > 0 )
@@ -129,17 +139,6 @@ PackageSelector::PackageSelector(Package* pkg, QWidget *parent) :
 		}
 	}
 
-	if( pkg->markedOperation() == Package::PkgOpInstallUpgrade )
-	{
-		if( pkg->isUpgradeable() )
-			longtext += "Package has been marked for <b>upgrade</b><br>";
-		else
-			longtext += "Package has been marked for <b>installation</b><br>";
-	} else if(pkg->markedOperation() == Package::PkgOpRemove)
-	{
-		longtext += "Package has been marked for <b>removal</b><br>";
-	}
-
 	longtext += "<br>";
 	longtext += pkg->descShort();
 	QString descLong = pkg->descLong();
@@ -176,17 +175,7 @@ PackageSelector::PackageSelector(Package* pkg, QWidget *parent) :
 		ui->radioRemove->setChecked(true);
 	}
 
-	if( pkg->isInstalled() && !pkg->isUpgradeable() )
-	{
-		ui->radioInstall->setEnabled(false);
-		ui->radioInstall->hide();
-	}
-	if( !pkg->isInstalled() ) {
-		ui->radioRemove->setEnabled(false);
-		ui->radioRemove->hide();
-	}
-
-#ifndef Q_WS_MAEMO_5	// for simulator
+#ifndef Q_WS_MAEMO_5	// for simulator debug
 	ui->radioInstall->setChecked(true);
 #endif
 }
@@ -217,3 +206,106 @@ Package::operation PackageSelector::selectedOperation()
 
 	return Package::PkgOpNone;
 }
+
+void PackageSelector::on_pushButton_blacklist_clicked()
+{
+	BlacklistSelect s(iPkg, this);
+	BlacklistSelect::blackList old = iPkg->blacklisted();
+	if( s.exec() ) {
+		updateHeader2();
+		if( !iPkg->isBlacklisted() && old != iPkg->blacklisted() ) {
+			iAptInterface->removeFromBlacklist(iPkg, old);
+			iAptInterface->writeBlacklist();
+		}
+	}
+}
+
+QString PackageSelector::getMaemoOrgUrl(Package* pkg)
+{
+	QString url = "";
+
+	if( !pkg )
+		return url;
+
+	if( pkg->repositories().count() == 0 )
+		return url;
+
+	for( int i=0; i<pkg->repositories().count(); i++ )
+	{
+		if( pkg->repositories().at(i) && pkg->repositories().at(i)->url().startsWith("http://repository.maemo.org") )
+			url = "http://maemo.org/packages/view/" + pkg->name() + "/";
+	}
+	return url;
+}
+
+void PackageSelector::updateHeader2()
+{
+	QString header2;
+
+	if( iPkg->isInstalled() ) {
+		header2 += "Installed, version <b>" + iPkg->version() + "</b>";
+	} else {
+		header2 += "Not installed";
+	}
+	header2 += "<br>";
+
+	if( iPkg->markedOperation() == Package::PkgOpInstallUpgrade )
+	{
+		if( iPkg->isUpgradeable() )
+			header2 += "Marked for <b>upgrade</b>";
+		else
+			header2 += "Marked for <b>installation</b>";
+	} else if(iPkg->markedOperation() == Package::PkgOpRemove)
+	{
+		header2 += "Marked for <b>removal</b>";
+	}
+
+	if(iPkg->isBlacklisted()) {
+		header2 = "BLACKLISTED";
+		if( iPkg->blacklisted() == BlacklistSelect::BlacklistAll )
+			header2 += " (all)";
+		else if( iPkg->blacklisted() == BlacklistSelect::BlacklistThis )
+			header2 += " (this)";
+		iPkg->setMarkedForOperation(Package::PkgOpNone);
+	}
+
+	ui->radioInstall->setEnabled(true);
+	ui->radioRemove->setEnabled(true);
+	ui->radioInstall->show();
+	ui->radioRemove->show();
+
+	if( iPkg->isInstalled() && !iPkg->isUpgradeable() )
+	{
+		ui->radioInstall->setEnabled(false);
+		ui->radioInstall->hide();
+	} else if( !iPkg->isInstalled() ) {
+		ui->radioRemove->setEnabled(false);
+		ui->radioRemove->hide();
+	}
+
+	if( iPkg->isBlacklisted() ) {
+		ui->radioInstall->setEnabled(false);
+		ui->radioRemove->setEnabled(false);
+	}
+
+	ui->label_header2->setText(header2);
+}
+
+void PackageSelector::on_pushButton_website_clicked()
+{
+	QString url = getMaemoOrgUrl(iPkg);
+	if( url == "" ) {
+		url = getMaemoOrgUrl(iPkg->availablePackage());
+		if( url == "")
+			return;
+	}
+
+	QDBusConnection conn = QDBusConnection::connectToBus(QDBusConnection::SessionBus, "faster_application_manager");
+
+	QString service = "com.nokia.osso_browser";
+	QString path = "/com/nokia/osso_browser/request";
+	QString method = "open_new_window";
+
+	QDBusInterface browser(service, path, service, conn, this);
+	browser.call(method, url);
+}
