@@ -27,12 +27,14 @@ namespace Ui {
     class PackageSelector;
 }
 
+class AAptInterface;
+
 class PackageSelector : public QDialog
 {
     Q_OBJECT
 
 public:
-	explicit PackageSelector(Package* pkg, QWidget *parent = 0);
+	explicit PackageSelector(Package* pkg, AAptInterface* apt, QWidget *parent);
     ~PackageSelector();
 
 #ifdef Q_WS_MAEMO_5		// workaround for buggy Qt autorotation
@@ -53,7 +55,16 @@ protected:
     void changeEvent(QEvent *e);
 
 private:
+	QString getMaemoOrgUrl(Package* pkg);
+	void updateHeader2();
+
     Ui::PackageSelector *ui;
+	Package* iPkg;
+	AAptInterface* iAptInterface;
+
+private slots:
+	void on_pushButton_website_clicked();
+	void on_pushButton_blacklist_clicked();
 };
 
 #endif // PACKAGESELECTOR_H
