@@ -17,7 +17,6 @@
 	(C) Heikki Holstila 2010
 */
 
-#include <iostream>
 #include "packageview.h"
 #include "ui_packageview.h"
 #include "package.h"
@@ -169,7 +168,8 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 			<< tr("\tScience")
 			<< tr("\tSystem")
 			<< tr("\tUtilities")
-			<< tr("All packages (ADVANCED)");
+			<< tr("All packages (ADVANCED)")
+			<< tr("Blacklisted packages");	// a special case
 
 	iCatFilterStrings
 			<< ""
@@ -187,6 +187,7 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 			<< "user/science"
 			<< "user/system"
 			<< "user/utilities"
+			<< ""
 			<< "";
 
 	iDefaultCatFilter = CatFilterAllUser;
@@ -212,7 +213,7 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 	iKeyFilter = new KeyEventGrabber(this);
 	ui->listWidget->installEventFilter(iKeyFilter);
 
-
+	iSortNoticeShown = false;
 }
 
 PackageView::~PackageView()
@@ -240,6 +241,13 @@ bool PackageView::doFilterCategory(Package* pkg)
 	if( pkg->section()=="user/hidden" )
 		return false;
 
+	if( pkg->isBlacklisted() && iSelectedCatFilter != CatFilterBlacklisted )
+		return false;
+	else if ( pkg->isBlacklisted() && iSelectedCatFilter == CatFilterBlacklisted )
+		return true;
+	if( iSelectedCatFilter == CatFilterBlacklisted )
+		return false;
+
 	if( iSelectedCatFilter==CatFilterAllMarked ) {
 		if( pkg->isMarkedForOperation() )
 			return true;
@@ -258,9 +266,15 @@ QString PackageView::generateSortString(Package* pkg)
 	QString sortstr = pkg->displayName();
 
 	if( iSortOrder==SortDateDesc ) {
-		if( pkg->date().isValid() )
+		Package* upg = 0;
+		if( pkg->isUpgradeable() )
+			upg = pkg->availablePackage();
+
+		if( !upg && pkg->date().isValid() )
 			sortstr = pkg->date().toString("yyyy-MM-dd hh:mm");
-		else {
+		else if( upg && upg->date().isValid() ) {
+			sortstr = pkg->date().toString("yyyy-MM-dd hh:mm");
+		} else {
 			sortstr = "";
 			iPackagesEmptySort++;
 		}
@@ -287,7 +301,6 @@ void PackageView::openWin()
 		ui->listWidget->sortItems(Qt::DescendingOrder);
 	else
 		ui->listWidget->sortItems(Qt::AscendingOrder);
-	ui->btn_Sort->hide();
 
 	delete ui->listWidget->itemDelegate();
 	ListItemDelegate* delegate = new ListItemDelegate(ui->listWidget);
@@ -297,7 +310,7 @@ void PackageView::openWin()
 	if( !ui->searchBar->isVisible() )
 	{
 		if( iSelectedStatFilter == Package::PkgStatNotInstalled || iSelectedStatFilter == Package::PkgStatUnknown ||
-			iSelectedCatFilter == CatFilterAllMarked )
+			iSelectedCatFilter == CatFilterAllMarked || iSelectedCatFilter == CatFilterBlacklisted )
 		{
 			QHashIterator<QString, Package*> i( *iAptInterface->packagesAvailable() );
 			while (i.hasNext())
@@ -309,7 +322,8 @@ void PackageView::openWin()
 			}
 		}
 		if( iSelectedStatFilter == Package::PkgStatInstalled || iSelectedStatFilter == Package::PkgStatUpgradeable ||
-			iSelectedStatFilter == Package::PkgStatUnknown || iSelectedCatFilter == CatFilterAllMarked )
+			iSelectedStatFilter == Package::PkgStatUnknown || iSelectedCatFilter == CatFilterAllMarked ||
+			iSelectedCatFilter == CatFilterBlacklisted )
 		{
 			QHashIterator<QString, Package*> i( *iAptInterface->packagesInstalled() );
 			while (i.hasNext())
@@ -336,10 +350,19 @@ void PackageView::openWin()
 
 	updateLabel();
 
-	if( iSelectedStatFilter==Package::PkgStatUpgradeable && ui->listWidget->count()>0 )
+	if( iSelectedStatFilter==Package::PkgStatUpgradeable && ui->listWidget->count()>0 &&
+		iSelectedCatFilter != CatFilterAllMarked && iSelectedCatFilter != CatFilterBlacklisted )
+	{
 		ui->actionUpgrade_all->setVisible(true);
-	else
+	} else {
 		ui->actionUpgrade_all->setVisible(false);
+	}
+
+	if( iSelectedCatFilter == CatFilterBlacklisted && ui->listWidget->count()>0 ) {
+		ui->actionRestore_all->setVisible(true);
+	} else {
+		ui->actionRestore_all->setVisible(false);
+	}
 
 	show();
 
@@ -358,10 +381,11 @@ void PackageView::openWin()
 		iListCoverLabel->show();
 	}
 
-	if( ui->listWidget->count()>0 && iPackagesEmptySort == ui->listWidget->count() ) {
+	if( ui->listWidget->count()>0 && iPackagesEmptySort == ui->listWidget->count() && !iSortNoticeShown ) {
 		ConfirmDialog d(false, this);
-		d.setText("Warning", "No packages have the required information for sorting by this criterion. Leaving list unsorted.");
+		d.setText("Notice", "No shown packages currently have the required information for sorting by this criterion. The list is unsorted.");
 		d.exec();
+		iSortNoticeShown = true;
 	}
 }
 
@@ -524,13 +548,22 @@ void PackageView::clearSelections()
 
 void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 {
-	PackageSelector s(dynamic_cast<PackageListWidgetItem*>(item)->package(), this);
+	Package* pkg = dynamic_cast<PackageListWidgetItem*>(item)->package();
+	if( !pkg )
+		return;
+
+	bool bl = pkg->isBlacklisted();
+
+	PackageSelector s(pkg, iAptInterface, this);
 	s.exec();
 
-	dynamic_cast<PackageListWidgetItem*>(item)->package()->setMarkedForOperation( s.selectedOperation() );
-	item->setData(UserRoleMarked, (int)s.selectedOperation());
-
-	updateLabel();
+	if( pkg->isBlacklisted() != bl ) {
+		openWin();
+	} else {
+		pkg->setMarkedForOperation( s.selectedOperation() );
+		item->setData(UserRoleMarked, (int)s.selectedOperation());
+		updateLabel();
+	}
 }
 
 void PackageView::updateLabel()
@@ -540,7 +573,7 @@ void PackageView::updateLabel()
 	QString s2;
 	s2.setNum( ui->listWidget->count() );
 	QString statlabel = iStatFilterLabels.at(iSelectedStatFilter);
-	if( iSelectedCatFilter == CatFilterAllMarked )
+	if( iSelectedCatFilter == CatFilterAllMarked || iSelectedCatFilter == CatFilterBlacklisted )
 		statlabel = "All";
 	ui->label->setText("<font size=\"-2\"><b>" + s + "</b> package(s) marked<br>"
 					   + "Showing: <b>" + statlabel + "</b><br>"
@@ -567,7 +600,7 @@ void PackageView::on_btn_CategoryFilter_clicked()
 	if( s )
 		iSelectedCatFilter = f.selection();
 
-	if( iSelectedCatFilter == CatFilterAllMarked ) {
+	if( iSelectedCatFilter == CatFilterAllMarked || iSelectedCatFilter == CatFilterBlacklisted ) {
 		ui->btn_StatusFilter->setEnabled(false);
 	} else {
 		ui->btn_StatusFilter->setEnabled(true);
@@ -578,6 +611,7 @@ void PackageView::on_btn_CategoryFilter_clicked()
 		iListCoverLabel->setAutoFillBackground(true);
 		iListCoverLabel->show();
 		QApplication::processEvents();
+		iSortNoticeShown = false;
 
 		openWin();
 	}
@@ -602,6 +636,7 @@ void PackageView::on_btn_StatusFilter_clicked()
 		iListCoverLabel->setAutoFillBackground(true);
 		iListCoverLabel->show();
 		QApplication::processEvents();
+		iSortNoticeShown = false;
 
 		openWin();
 	}
@@ -609,14 +644,14 @@ void PackageView::on_btn_StatusFilter_clicked()
 
 void PackageView::resetWindow()
 {
+	iAptInterface->writeBlacklist();
+
 	iSelectedCatFilter = iDefaultCatFilter;
 	ui->btn_StatusFilter->setEnabled(true);
 	iSortOrder = SortAlpha;
+	iSortNoticeShown = false;
 
 	clearSelections();
-	//if( iMainWindow )
-	//	iMainWindow->setOperation(MainWindow::OpNone);
-
 	clearSearch();
 }
 
@@ -663,6 +698,9 @@ void PackageView::on_actionSearch_triggered()
 
 void PackageView::on_lineEdit_textEdited(QString text)
 {
+	if( !ui->searchBar->isVisible() )
+		return;
+
 	if( text=="" ) {
 		on_btn_searchClose_clicked();
 		return;
@@ -678,23 +716,37 @@ void PackageView::on_lineEdit_textEdited(QString text)
 		ui->searchBar->hide();
 		openWin();
 		ui->searchBar->show();
+		ui->toolBar->hide(); // ensure it stays hidden
 		ui->lineEdit->setFocus();
 	}
 
 	iPrevSearchText = text;
-	QList<QListWidgetItem*> results;
-	if( text.startsWith(":") )
-		results = ui->listWidget->findItems( text.mid(1), Qt::MatchStartsWith );
-	else
-		results = ui->listWidget->findItems( text, Qt::MatchContains );
-
-	ui->searchLabel->setText( QString("%1 results").arg(results.count()) );
 
+	QList<Package*> packages;
 	iSearchResults.clear();
-	for( int i=0; i<results.count(); i++ )
-	{
-		iSearchResults.append( dynamic_cast<PackageListWidgetItem*>(results.at(i))->package() );
+
+	for( int i=0; i<(ui->listWidget->count()); i++ ) {
+		packages.append( dynamic_cast<PackageListWidgetItem*>( ui->listWidget->item(i) )->package() );
 	}
+
+	if( text.startsWith(":") ) {
+		for( int i=0; i<packages.count(); i++ ) {
+			if( packages.at(i) && ( packages.at(i)->displayName().startsWith(text.mid(1), Qt::CaseInsensitive) ))
+			{
+				iSearchResults.append( packages.at(i) );
+			}
+		}
+	} else {
+		for( int i=0; i<packages.count(); i++ ) {
+			if( packages.at(i) && ( packages.at(i)->displayName().contains(text, Qt::CaseInsensitive) ))
+			{
+				iSearchResults.append( packages.at(i) );
+			}
+		}
+	}
+
+	ui->searchLabel->setText( QString("%1 results").arg( iSearchResults.count()) );
+
 	openWin();
 }
 
@@ -727,7 +779,7 @@ void PackageView::on_actionUpgrade_all_triggered()
 	for( int i=0; i<ui->listWidget->count(); i++ )
 	{
 		Package* pkg = dynamic_cast<PackageListWidgetItem*>(ui->listWidget->item(i))->package();
-		if( pkg->isUpgradeable() ) {
+		if( pkg && pkg->isUpgradeable() ) {
 			pkg->setMarkedForOperation( Package::PkgOpInstallUpgrade );
 			ui->listWidget->item(i)->setData(UserRoleMarked, (int)Package::PkgOpInstallUpgrade);
 		}
@@ -747,37 +799,201 @@ void PackageView::on_btn_Sort_clicked()
 	SortSelector s(iSortOrder, this);
 	if( s.exec() ) {
 		iSortOrder = s.selectedOperation();
+		iSortNoticeShown = false;
 		openWin();
 	}
 }
 
 void PackageView::on_actionLoad_selections_triggered()
 {
+	if( iAptInterface->numSelectedPackages() > 0 ) {
+		ConfirmDialog d(true, this);
+		d.setText("Confirmation", "Proceed loading selections? All current selections will be cleared.");
+		if( !d.exec() )
+			return;
+	}
 
+	clearSelections();
+
+	QStringList unknownList;
+	QStringList wrongverList;
+	int success=0;
+	int errors=0;
+
+	QFile f("/root/.fapman/selections.list");
+	if( f.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while(!f.atEnd()) {
+			QString line = f.readLine().trimmed();
+			QStringList parts = line.split(' ');
+			if( parts.count()==3 ) {
+				Package* pkgAv = iAptInterface->packagesAvailable()->value(parts.at(0),0);
+				Package* pkgIn = iAptInterface->packagesInstalled()->value(parts.at(0),0);
+				if( parts.at(2)=="install" ) {
+					if( !pkgAv && !pkgIn ) {
+						unknownList << parts.at(0);
+					}
+					else if( pkgIn ) {
+						wrongverList << parts.at(0);
+					}
+					else if( pkgAv ) {
+						if( pkgAv->version() == parts.at(1) && !pkgAv->isInstalled() ) {
+							pkgAv->setMarkedForOperation(Package::PkgOpInstallUpgrade);
+							success++;
+						} else {
+							wrongverList << parts.at(0);
+						}
+					}
+
+				} else if( parts.at(2)=="upgrade" ) {
+					if( !pkgAv && !pkgIn ) {
+						unknownList << parts.at(0);
+					}
+					else if( (pkgAv && !pkgIn) || (pkgIn && !pkgAv) ) {
+						wrongverList << parts.at(0);
+					}
+					else if( pkgIn && pkgAv ) {
+						if( pkgIn->version() == parts.at(1) && pkgIn->isInstalled() && pkgIn->isUpgradeable() ) {
+							pkgIn->setMarkedForOperation(Package::PkgOpInstallUpgrade);
+							success++;
+						} else {
+							wrongverList << parts.at(0);
+						}
+					}
+
+				} else if( parts.at(2)=="remove" ) {
+					if( !pkgAv && !pkgIn ) {
+						unknownList << parts.at(0);
+					}
+					else if( pkgAv && !pkgIn ) {
+						wrongverList << parts.at(0);
+					}
+					else if( pkgIn ) {
+						if( pkgIn->version() == parts.at(1) && pkgIn->isInstalled() ) {
+							pkgIn->setMarkedForOperation(Package::PkgOpRemove);
+							success++;
+						} else {
+							wrongverList << parts.at(0);
+						}
+					}
+				} else {
+					errors++;
+				}
+			} else if( line!="" ){
+				errors++;
+			}
+		}
+		f.close();
+	}
+
+	ConfirmDialog d(false, this);
+	QString msg;
+	msg += QString("<b>%1 successful</b><br>").arg(success);
+	if( wrongverList.count() > 0 ) {
+		msg += QString("%1 wrong versions/changed statuses (not selected):<br>").arg(wrongverList.count());
+		msg += "<font size=\"-1\">";
+		for( int i=0; i<wrongverList.count(); i++ ) {
+			msg += wrongverList.at(i) + " ";
+		}
+		msg += "</font><br>";
+	}
+	if( unknownList.count() > 0 ) {
+		msg += QString("%1 unknown packages:<br>").arg(unknownList.count());
+		msg += "<font size=\"-1\">";
+		for( int i=0; i<unknownList.count(); i++ ) {
+			msg += unknownList.at(i) + " ";
+		}
+		msg += "</font><br>";
+	}
+	if( errors>0 || (wrongverList.count()==0 && unknownList.count()==0) ) {
+		msg += QString("%1 errors<br>").arg(errors);
+	}
+	if( success==0 && wrongverList.count()==0 && unknownList.count()==0 )
+		msg = "No stored selections";
+	QString title;
+	if( success > 0 )
+		title = "Selections loaded";
+	else
+		title = "No selections loaded";
+	d.setText(title, msg);
+	d.exec();
+
+	if( success > 0 ) {
+		ui->btn_StatusFilter->setEnabled(false);
+		iSelectedCatFilter = CatFilterAllMarked;
+	}
+	openWin();
 }
 
 void PackageView::on_actionSave_selections_triggered()
 {
-	QHashIterator<QString, Package*> i( *iAptInterface->packagesAvailable() );
-	while (i.hasNext())
+	if( iAptInterface->numSelectedPackages() == 0 )
+		return;
+
+	QFile f("/root/.fapman/selections.list");
+	bool fail = false;
+	int count = 0;
+	if( f.open(QIODevice::WriteOnly | QIODevice::Text) )
 	{
-		i.next();
+		QTextStream out(&f);
 
-		if( i.value()->markedOperation() == Package::PkgOpInstallUpgrade )
-			qDebug() << i.value()->name() << i.value()->version() << "install";
-		if( i.value()->markedOperation() == Package::PkgOpRemove )
-			qDebug() << "Warning: package is marked for removal in the wrong list";
+		QHashIterator<QString, Package*> i( *iAptInterface->packagesAvailable() );
+		while (i.hasNext())
+		{
+			i.next();
 
+			if( i.value()->markedOperation() == Package::PkgOpInstallUpgrade ) {
+				out << i.value()->name() << " " << i.value()->version() << " install\n";
+				count++;
+			}
+			if( i.value()->markedOperation() == Package::PkgOpRemove )
+				qDebug() << "Warning: package is marked for removal in the wrong list";
+		}
+
+		QHashIterator<QString, Package*> r( *iAptInterface->packagesInstalled() );
+		while (r.hasNext())
+		{
+			r.next();
+
+			if( r.value()->markedOperation() == Package::PkgOpInstallUpgrade ) {
+				out << r.value()->name() << " " << r.value()->version() << " upgrade\n";
+				count++;
+			}
+			if( r.value()->markedOperation() == Package::PkgOpRemove ) {
+				out << r.value()->name() << " " << r.value()->version() << " remove\n";
+				count++;
+			}
+		}
+
+		f.close();
+	} else {
+		fail = true;
 	}
 
-	QHashIterator<QString, Package*> r( *iAptInterface->packagesInstalled() );
-	while (r.hasNext())
+	ConfirmDialog d(false, this);
+	if( fail )
+		d.setText( "Error", "Failed to write package selections" );
+	else
+		d.setText( "Selections stored", QString("Stored %1 selections").arg(count) );
+	d.exec();
+}
+
+void PackageView::on_actionRestore_all_triggered()
+{
+	ConfirmDialog d(true, this);
+	d.setText("Confirmation","Restore all shown blacklisted packages?");
+	if( !d.exec() )
+		return;
+
+	for( int i=0; i<ui->listWidget->count(); i++ )
 	{
-		r.next();
-
-		if( r.value()->markedOperation() == Package::PkgOpInstallUpgrade )
-			qDebug() << i.value()->name() << i.value()->version() << "upgrade";
-		if( r.value()->markedOperation() == Package::PkgOpRemove )
-			qDebug() << i.value()->name() << i.value()->version() << "remove";
+		Package* pkg = dynamic_cast<PackageListWidgetItem*>(ui->listWidget->item(i))->package();
+		if( pkg && pkg->isBlacklisted() ) {
+			BlacklistSelect::blackList old = pkg->blacklisted();
+			pkg->setBlacklisted(BlacklistSelect::BlacklistNone);
+			iAptInterface->removeFromBlacklist(pkg, old);
+		}
 	}
+	iAptInterface->writeBlacklist();
+	openWin();
 }
