@@ -143,10 +143,12 @@ private:
 	int iDefaultCatFilter;
 	static const int CatFilterAllMarked = 0;
 	static const int CatFilterAllUser = 1;
+	static const int CatFilterBlacklisted = 16;
 
 	QStringList iStatFilterLabels;
 	int iSelectedStatFilter;
 	sortOrder iSortOrder;
+	bool iSortNoticeShown;
 
 	QList<Package*> iSearchResults;
 	QString iPrevSearchText;
@@ -155,7 +157,8 @@ private:
 	int iPackagesEmptySort;
 
 private slots:
-	void on_actionSave_selections_triggered();
+	void on_actionRestore_all_triggered();
+ void on_actionSave_selections_triggered();
 	void on_actionLoad_selections_triggered();
 	void on_btn_Sort_clicked();
 	void on_actionView_log_triggered();
