@@ -51,19 +51,29 @@ void Settings::changeEvent(QEvent *e)
 
 void Settings::openWin()
 {
+	ui->checkBox_fetchdates->setChecked( iQSettings->value("fetch_dates", false).toBool() );
 	ui->checkBox_autorotation->setChecked( iQSettings->value("disable_autorotation", false).toBool() );
 	ui->checkBox_autoremove->setChecked( iQSettings->value("enable_autoremove", false).toBool() );
 	ui->checkBox_autoclean->setChecked( iQSettings->value("enable_autoclean", false).toBool() );
 
+	ui->checkBox_proxies->setChecked( iQSettings->value("use_proxies", false).toBool() );
+	ui->lineEdit_http_proxy->setText(iQSettings->value("http_proxy","").toString() );
+	ui->lineEdit_https_proxy->setText(iQSettings->value("https_proxy","").toString() );
+
 	open();
 }
 
 void Settings::on_btn_OK_clicked()
 {
+	iQSettings->setValue("fetch_dates", ui->checkBox_fetchdates->isChecked() );
 	iQSettings->setValue("disable_autorotation", ui->checkBox_autorotation->isChecked() );
 	iQSettings->setValue("enable_autoremove", ui->checkBox_autoremove->isChecked() );
 	iQSettings->setValue("enable_autoclean", ui->checkBox_autoclean->isChecked() );
 
+	iQSettings->setValue("use_proxies", ui->checkBox_proxies->isChecked() );
+	iQSettings->setValue("http_proxy", ui->lineEdit_http_proxy->text() );
+	iQSettings->setValue("https_proxy", ui->lineEdit_https_proxy->text() );
+
 #ifdef Q_WS_MAEMO_5
 	if( !iQSettings->value("disable_autorotation",false).toBool() ) {
 		dynamic_cast<MainWindow*>(this->parent())->setAttribute(Qt::WA_Maemo5AutoOrientation);
